"use strict";(self.webpackChunk_streamlit_app=self.webpackChunk_streamlit_app||[]).push([[4500],{54500:(t,e,l)=>{l.r(e),l.d(e,{default:()=>r});var s=l(90994),i=(l(66845),l(40864));function r(t){let{element:e,width:l}=t;const r=e.hasWidth?e.width:l;let n,o;e.scrolling?(n="auto",o={}):(n="no",o={overflow:"hidden"}),o.colorScheme="light dark";const c=a(e.src),d=null!=c?void 0:a(e.srcdoc);return(0,i.jsx)("iframe",{"data-testid":"stIFrame",allow:s.p,style:o,src:c,srcDoc:d,width:r,height:e.height,scrolling:n,sandbox:s.T,title:"st.iframe"})}function a(t){return null==t||""===t?void 0:t}}}]);